#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# wls_install::bsu
#
# @summary This defined type allows you add or remove BSU patches to a WebLogic system.
# Here is an example on how you could use this:
# 
# ```puppet
# orawls::bsu {'BYJ1':
#   ensure                  => "present",
#   middleware_home_dir     => "/opt/oracle/middleware11gR1",
#   weblogic_home_dir       => "/opt/oracle/middleware11gR1/wlserver",
#   jdk_home_dir            => "/usr/java/jdk1.7.0_45",
#   patch_id                => "BYJ1",
#   patch_file              => "p17071663_1036_Generic.zip",
#   os_user                 => "oracle",
#   os_group                => "dba",
#   download_dir            => "/data/install",
#   source                  => "/vagrant",
# }
# ```
#
# @param [Enum[present, absent]] ensure
#    State to obtain.
#    The ensure attribute can be one of two values:
#    - present
#    - absent
#    When you specify `present`, Puppet will make sure the resource is available with all specified options and properties.
#    When the resource is already available(installed), and all attributes are as the are specified, Puppet will do nothing.
#    When you specify `absent`, Puppet will remove the resource if it is available. If it is not installed, Puppet will do nothing.
#
# @param [Variant[Integer[1036, 1036], Integer[1111, 1111], Integer[1211, 1211]]] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] patch_id
#    Patch id to apply.
#
# @param [String[1]] patch_file
#    File containing the BSU patch.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] source
#    Source for the operation.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Variant[Boolean, Enum['on_failure']]]] log_output
#    **DEPRECATED** This parameter is replaced by 'logoutput' and will be removed in a future release.
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Boolean]] remote_file
#    **DEPRECATED**
#    This parameter is not being used anymore and will be removed in a future release.
#    `true`.
#
#--++--
define wls_install::bsu (
  Enum[
    present,
    absent
  ]                    $ensure              = 'present',  #present|absent
  Variant[
    Integer[1036,1036],
    Integer[1111,1111],
    Integer[1211,1211]
  ]                    $version             = lookup({name => 'wls_version', default_value => 1036}),
  Stdlib::Absolutepath $middleware_home_dir = lookup({name => 'wls_middleware_home_dir', default_value => undef}),
  Stdlib::Absolutepath $weblogic_home_dir   = lookup({name => 'wls_weblogic_home_dir', default_value => undef}),
  Stdlib::Absolutepath $jdk_home_dir        = lookup({name => 'wls_jdk_home_dir', default_value => undef}),
  String[1]            $patch_id            = undef,
  String[1]            $patch_file          = undef,
  String[1]            $os_user             = lookup({name => 'wls_os_user', default_value => undef}),
  String[1]            $os_group            = lookup({name => 'wls_os_group', default_value => undef}),
  Stdlib::Absolutepath $download_dir        = lookup({name => 'wls_download_dir', default_value => undef}),
  String[1]            $source              = lookup({name => 'wls_source', default_value => undef}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput           = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum['on_failure']
    ]
  ]                    $log_output          = undef, # DEPRECATED
  Optional[Boolean]    $remote_file         = undef, # DEPRECATED
)
{
  warning 'This class is deprecated and will be removed in a future release'
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  $exec_path = "/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:${jdk_home_dir}/bin"

  if ( $source ) {
    $mountPoint = $source
  }
  else {
    $mountPoint = 'puppet:///modules/wls_install/'
  }

  unless ( defined(File["${middleware_home_dir}/utils/bsu/cache_dir"]) ) {
    file { "${middleware_home_dir}/utils/bsu/cache_dir":
      ensure => directory,
      mode   => '0775',
      owner  => $os_user,
      group  => $os_group,
    }
  }

  if ( $ensure == 'present' ) {
    # the patch used by the bsu
    archive { "${download_dir}/${patch_file}":
      ensure  => present,
      cleanup => false,
      extract => false,
      source  => "${mountPoint}/${patch_file}",
      user    => $os_user,
      group   => $os_group,
      require => File["${middleware_home_dir}/utils/bsu/cache_dir"],
    }
    $disk1_file = "${download_dir}/${patch_file}"

    if ( $version == 1111 ) {
      $patch_version = 1036
    } else {
      $patch_version = $version
    }

    exec { "patch policy for ${patch_file}":
      command   => "bash -c \"{ echo 'grant codeBase \\\"file:${middleware_home_dir}/patch_wls1036/patch_jars/-\\\" {'; echo '      permission java.security.AllPermission;'; echo '};'; } >> ${weblogic_home_dir}/server/lib/weblogic.policy\"",
      unless    => "grep 'file:${middleware_home_dir}/patch_wls1036/patch_jars/-' ${weblogic_home_dir}/server/lib/weblogic.policy",
      path      => $exec_path,
      user      => $os_user,
      group     => $os_group,
      logoutput => $logoutput,
      require   => Bsu_patch[$patch_id],
    }
  }

  bsu_patch{ $patch_id:
    ensure              => $ensure,
    os_user             => $os_user,
    middleware_home_dir => $middleware_home_dir,
    weblogic_home_dir   => $weblogic_home_dir,
    jdk_home_dir        => $jdk_home_dir,
  }
}
